@@ -1,11 +1,12 @@
 package Hijk;
 use strict;
 use warnings;
+use Time::HiRes;
 use POSIX qw(:errno_h);
 use Socket qw(PF_INET SOCK_STREAM pack_sockaddr_in inet_ntoa $CRLF SOL_SOCKET SO_ERROR);
 use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
 
-our $VERSION = "0.20";
+our $VERSION = "0.21";
 
 sub Hijk::Error::CONNECT_TIMEOUT         () { 1 << 0 } # 1
 sub Hijk::Error::READ_TIMEOUT            () { 1 << 1 } # 2
@@ -32,7 +33,7 @@ sub _read_http_message {
     vec(my $rin = '', $fd, 1) = 1;
     do {
         return ($close_connection,undef,0,undef,undef, Hijk::Error::READ_TIMEOUT)
-            if ((select($rin, undef, undef, $read_timeout) != 1) || (defined($read_timeout) && $read_timeout <= 0));
+            if ((_select($rin, undef, undef, $read_timeout) != 1) || (defined($read_timeout) && $read_timeout <= 0));
 
         my $nbytes = POSIX::read($fd, $buf, $read_length);
         return ($close_connection, $proto, $status_code, $header, $body)
@@ -152,7 +153,7 @@ sub _read_chunked_body {
         # just read a 10k block and process it until it is consumed
         if (length($buf) == 0 || length($buf) < $chunk_size) {
             return (undef, Hijk::Error::READ_TIMEOUT)
-                if ((select($rin, undef, undef, $read_timeout) != 1) || (defined($read_timeout) && $read_timeout <= 0));
+                if ((_select($rin, undef, undef, $read_timeout) != 1) || (defined($read_timeout) && $read_timeout <= 0));
             my $current_buf = "";
             my $nbytes = POSIX::read($fd, $current_buf, $read_length);
             if (!defined($nbytes)) {
@@ -248,7 +249,7 @@ sub _construct_socket {
 
     $connect_timeout = undef if defined($connect_timeout) && $connect_timeout <= 0;
     vec(my $rout = '', fileno($soc), 1) = 1;
-    if (select(undef, $rout, undef, $connect_timeout) != 1) {
+    if (_select(undef, $rout, undef, $connect_timeout) != 1) {
         if (defined($connect_timeout)) {
             return (undef, {error => Hijk::Error::CONNECT_TIMEOUT});
         } else {
@@ -278,15 +279,14 @@ sub _build_http_message {
         $CRLF,
         ($args->{method} || "GET")." $path_and_qs " . ($args->{protocol} || "HTTP/1.1"),
         "Host: $args->{host}",
-        $args->{body} ? ("Content-Length: " . length($args->{body})) : (),
+        defined($args->{body}) ? ("Content-Length: " . length($args->{body})) : (),
         $args->{head} ? (
             map {
                 $args->{head}[2*$_] . ": " . $args->{head}[2*$_+1]
             } 0..$#{$args->{head}}/2
         ) : (),
-        "",
-        $args->{body} ? $args->{body} : ()
-    ) . $CRLF;
+        ""
+    ) . $CRLF . (defined($args->{body}) ? $args->{body} : "");
 }
 
 our $SOCKET_CACHE = {};
@@ -325,7 +325,7 @@ sub request {
 
     vec(my $rout = '', fileno($soc), 1) = 1;
     while ($left > 0) {
-        if (select(undef, $rout, undef, undef) != 1) {
+        if (_select(undef, $rout, undef, undef) != 1) {
             delete $args->{socket_cache}->{$cache_key} if defined $cache_key;
             return {
                 error         => Hijk::Error::REQUEST_SELECT_ERROR,
@@ -386,6 +386,19 @@ sub request {
     };
 }
 
+sub _select {
+    my ($rbits, $wbits, $ebits, $timeout) = @_;
+    while (1) {
+        my $start = Time::HiRes::time();
+        my $nfound = select($rbits, $wbits, $ebits, $timeout);
+        if ($nfound == -1 && $! == EINTR) {
+            $timeout -= Time::HiRes::time() - $start if $timeout;
+            next;
+        }
+        return $nfound;
+    }
+}
+
 1;
 
 __END__
